/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor;

import com.sun.jersey.api.client.GenericType;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.core.header.MediaTypes;
import com.sun.jersey.test.framework.JerseyTest;
import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.webidor.resources.Games;
import cz.xelfi.quoridor.webidor.resources.Quoridor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import javax.ws.rs.core.MediaType;
import org.junit.Test;
import org.w3c.dom.Document;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class QuoridorTest extends JerseyTest {
    private File dir;

    public QuoridorTest() throws Exception {
        super("cz.xelfi.quoridor.webidor.resources");
    }

    @Override
    public void setUp() throws Exception {
        dir = File.createTempFile("quoridor", ".dir");
        dir.delete();
        System.setProperty("quoridor.dir", dir.getPath());
        dir.mkdirs();
        File passwd = new File(dir, "passwd");
        FileOutputStream os = new FileOutputStream(passwd);
        os.write("Jarda=heslo\nJirka=pesko\n".getBytes("UTF-8"));
        os.close();
        super.setUp();
    }

    @Override
    public void tearDown() throws Exception {
        super.tearDown();
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }

    /**
     * Test if a WADL document is available at the relative path
     * "application.wadl".
     */
    @Test public void testApplicationWadl() {
        String serviceWadl = webResource.path("application.wadl").
                accept(MediaTypes.WADL).get(String.class);
        assertTrue(serviceWadl.length() > 0);
    }

    @Test public void testCreateAGame() throws Exception {
        webResource = webResource.path("api");
        String logJarda = webResource.path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = webResource.path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
    GameId s = webResource.path("games").queryParam("loginID", logJarda).
                queryParam("white", "Jarda")
                .queryParam("black", "Jirka").post(GameId.class);

        Thread.sleep(100);
        final long now = System.currentTimeMillis();
        if (s.getModified() >= now) {
            fail("The game is supposed to be modified in past");
        }
        Thread.sleep(100);

        String msg = webResource.path("games").get(String.class);
        //List<Game> games =  webResource.path("games").get(new GenericType<List<Game>>() {});

        GenericType<List<GameId>> gType = new GenericType<List<GameId>>() {};

        Document doc = webResource.path("games").accept("text/xml").get(Document.class);
        assertNotNull("Can read games in form of XML", doc);
        assertEquals("Name is correct", "gameIds", doc.getDocumentElement().getNodeName());
        assertEquals("One child", 1, doc.getDocumentElement().getChildNodes().getLength());
        List<GameId> games = webResource.path("games").accept("application/json").get(gType);
        assertEquals("One game", 1, games.size());
        assertEquals("Same white", "Jarda", games.get(0).getWhite());
        assertEquals("Same black", "Jirka", games.get(0).getBlack());

        GameId s1 = webResource.path("games/" + s.getId()).
            queryParam("loginID", logJarda).
            queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
        try {
            GameId s2 = webResource.path("games/" + s.getId()).
                queryParam("loginID", logJarda).
                queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
            fail("Not Jarda's turn, previous call shall fail");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        try {
            GameId s2 = webResource.path("games/" + s.getId()).
                queryParam("loginID", logJirka).
                queryParam("player", "Jirka").queryParam("move", "NONSENCE").put(GameId.class);
            fail("Invalid move");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        GameId s2 = webResource.path("games/" + s.getId()).
            queryParam("loginID", logJirka).
            queryParam("player", "Jirka").queryParam("move", "S").put(GameId.class);
        assertNotNull("Successful move", s2);
        if (s2.getModified() <= now) {
            fail("The game is newly modified");
        }
        Game snapshot = webResource.path("games/" + s.getId()).queryParam("move", "0").accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("All moves listed", 2, snapshot.getMoves().size());
        assertEquals("Current move", 0, snapshot.getCurrentMove());
        assertEquals("Position 0", 0, snapshot.getBoard().getPlayers().get(0).getRow());
        assertEquals("Position 8", 8, snapshot.getBoard().getPlayers().get(1).getRow());

        File game = new File(new File(dir, "games"), s2.getId());
        assertTrue("File for game exists", game.exists());

        char[] arr = new char[4096];
        FileReader gameContent = new FileReader(game);
        int len = gameContent.read(arr);
        String content = new String(arr, 0, len);

        if (!content.contains("# white: Jarda")) {
            fail(content);
        }
        if (!content.contains("# black: Jirka")) {
            fail(content);
        }
        if (!content.contains("N S")) {
            fail(content);
        }

        Games read = new Games(new File(dir, "games"), new Quoridor());
        List<Game> readGames = read.getGames();
        assertEquals("One game read", 1, readGames.size());
        Board board = readGames.get(0).getBoard();
        assertEquals(1, board.getPlayers().get(0).getRow());
        assertEquals(7, board.getPlayers().get(1).getRow());
        assertEquals(Move.NORTH, readGames.get(0).getMoves().get(0));
        assertEquals(Move.SOUTH, readGames.get(0).getMoves().get(1));

        class GMap extends GenericType<Map<String,Object>>{}
        String text = webResource.path("games").path(s.getId()).accept(MediaType.TEXT_PLAIN).get(String.class);
        if (text.indexOf("-----") == -1) {
            fail("Expecting board:\n" + text);
        }
        Game readGame = webResource.path("games").path(s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
        assertNotNull("Game really returned", readGame);
        assertEquals("Same game as in text representation", readGame.getBoard(), Board.valueOf(text));
        assertEquals("It is same as text of our game", readGame.getBoard().toString(), text);

        assertEquals(Move.NORTH, readGame.getMoves().get(0));
        assertEquals(Move.SOUTH, readGame.getMoves().get(1));
    }

}
